@@ -38,8 +38,11 @@
 #include <readline/readline.h>
 #include <readline/history.h>
 
-#include "src/shared/util.h"
+#include "lib/bluetooth.h"
+#include "lib/sdp.h"
 #include "lib/uuid.h"
+
+#include "src/shared/util.h"
 #include "btio/btio.h"
 #include "att.h"
 #include "gattrib.h"
