@@ -33,6 +33,9 @@ struct bthost;
 struct bthost *bthost_create(void);
 void bthost_destroy(struct bthost *bthost);
 
+typedef void (*bthost_ready_cb) (void);
+void bthost_notify_ready(struct bthost *bthost, bthost_ready_cb cb);
+
 void bthost_set_send_handler(struct bthost *bthost, bthost_send_func handler,
 							void *user_data);
 
@@ -81,6 +84,8 @@ void bthost_set_adv_enable(struct bthost *bthost, uint8_t enable,
 
 void bthost_write_ssp_mode(struct bthost *bthost, uint8_t mode);
 
+void bthost_write_le_host_supported(struct bthost *bthost, uint8_t mode);
+
 void bthost_request_auth(struct bthost *bthost, uint16_t handle);
 
 void bthost_le_start_encrypt(struct bthost *bthost, uint16_t handle,
@@ -91,6 +96,8 @@ typedef void (*bthost_l2cap_connect_cb) (uint16_t handle, uint16_t cid,
 void bthost_add_l2cap_server(struct bthost *bthost, uint16_t psm,
 				bthost_l2cap_connect_cb func, void *user_data);
 
+void bthost_set_sc_support(struct bthost *bthost, bool enable);
+
 void bthost_set_pin_code(struct bthost *bthost, const uint8_t *pin,
 							uint8_t pin_len);
 void bthost_set_io_capability(struct bthost *bthost, uint8_t io_capability);
@@ -100,6 +107,8 @@ uint8_t bthost_get_auth_req(struct bthost *bthost);
 void bthost_set_reject_user_confirm(struct bthost *bthost, bool reject);
 bool bthost_get_reject_user_confirm(struct bthost *bthost);
 
+uint64_t bthost_conn_get_fixed_chan(struct bthost *bthost, uint16_t handle);
+
 typedef void (*bthost_rfcomm_connect_cb) (uint16_t handle, uint16_t cid,
 						void *user_data, bool status);
 
@@ -129,9 +138,10 @@ void bthost_start(struct bthost *bthost);
 void *smp_start(struct bthost *bthost);
 void smp_stop(void *smp_data);
 void *smp_conn_add(void *smp_data, uint16_t handle, const uint8_t *ia,
-					const uint8_t *ra, bool conn_init);
+			const uint8_t *ra, uint8_t addr_type, bool conn_init);
 void smp_conn_del(void *conn_data);
 void smp_conn_encrypted(void *conn_data, uint8_t encrypt);
 void smp_data(void *conn_data, const void *data, uint16_t len);
+void smp_bredr_data(void *conn_data, const void *data, uint16_t len);
 int smp_get_ltk(void *smp_data, uint64_t rand, uint16_t ediv, uint8_t *ltk);
 void smp_pair(void *conn_data, uint8_t io_cap, uint8_t auth_req);
