@@ -30,16 +30,16 @@
 #include <inttypes.h>
 
 #include <glib.h>
-#include <gdbus/gdbus.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/rfcomm.h>
-#include <bluetooth/sdp.h>
-#include <bluetooth/sdp_lib.h>
+#include "lib/bluetooth.h"
+#include "lib/rfcomm.h"
+#include "lib/sdp.h"
+#include "lib/sdp_lib.h"
 
+#include "gdbus/gdbus.h"
 #include "btio/btio.h"
 
-#include "log.h"
+#include "obexd/src/log.h"
 #include "transport.h"
 #include "bluetooth.h"
 
